// https://eslint.org/docs/user-guide/configuring
// https://cn.eslint.org/docs/user-guide/configuring
// https://github.com/standard/standard/blob/master/docs/RULES-zhcn.md

/**
 * ESLint 附带有大量的规则。你可以使用注释或配置文件修改你项目中要使用的规则。要改变一个规则设置，你必须将规则 ID 设置为下列值之一：
 * "off" 或 0 - 关闭规则
 * "warn" 或 1 - 开启规则，使用警告级别的错误：warn (不会导致程序退出)
 * "error" 或 2 - 开启规则，使用错误级别的错误：error (当被触发的时候，程序会退出)
 */
module.exports = {
  root: true,
  parserOptions: {
    sourceType: 'module',
    parser: 'babel-eslint'
  },
  env: {
    browser: true,
  },
  extends: [
    'plugin:vue/essential',
    'standard'
  ],
  // required to lint *.vue files
  plugins: [
    'vue'
  ],
  rules: {
    // 一致的缩进，当前为 2 个空格：警告
    'indent': ['warn', 2],
    // 生产环境禁用 debugger：错误
    'no-debugger': process.env.NODE_ENV === 'production' ? 'error' : 'off',
    // function的左括号之前使用一致的空格：关闭
    'space-before-function-paren': 'off',
    // 多行空行：警告
    'no-multiple-empty-lines': 'warn',
    // 未使用过的变量：警告
    'no-unused-vars': 'warn',
    // 非骆驼拼写法命名约定：警告
    'camelcase': 'warn',
    // 不可达代码：警告
    'no-unreachable': 'warn',
    // 禁止在语句后使用分号：警告
    'semi': 'warn',
    // 禁用未声明的变量：警告
    'no-undef': 'warn',
    //禁止末尾逗号：警告
    'comma-dangle': 'warn',
    // 使用 === 和 !==：警告
    'eqeqeq': 'warn',
    // 禁用 tab
    'no-tabs': 'warn',
    // 禁止扩展原生类型
    'no-extend-native': 'off',
    // 行尾空格：警告
    'no-trailing-spaces': 'warn',
    // 禁止在 return 语句中使用赋值语句：警告
    'no-return-assign': 'warn',
    // 禁止不规则的空白
    'no-irregular-whitespace': 'warn',
    // 禁止空格和 tab 的混合缩进
    'no-mixed-spaces-and-tabs': 'warn',
    // 要求回调函数中有容错处理：警告
    'handle-callback-err': 'warn',
    'prefer-const': 'warn',
    'no-prototype-builtins': 'warn',
    'no-void': 'warn',
    'quotes': 'warn',
    // 使用过期函数：警告
    'node/no-deprecated-api': 'warn',
    // 未被使用的组件：警告
    'vue/no-unused-components': 'warn',
    // v-for中禁止使用v-if：警告
    'vue/no-use-v-if-with-v-for': 'warn',
    // v-for必须存在key：警告
    'vue/require-v-for-key': 'warn',
    // 属性赋值等号前后不能有空格：警告
    'vue/no-spaces-around-equal-signs-in-attribute': 'warn',
    // html 标签必须闭合：错误
    'vue/html-end-tags': 'error',
    // html 不要有多余的空格：警告
    'vue/no-multi-spaces': 'warn',
    // html 缩进为2个空格：警告
    'vue/html-indent': ['warn', 2, {
      "alignAttributesVertically": true
    }],
    // html 属性使用双引号：警告
    'vue/html-quotes': "warn",
    // html标签闭合尾标签前不能有空格：警告
    'vue/html-closing-bracket-spacing': 'warn',
    // 采用自闭合标签：关闭
    'vue/html-self-closing': 'off',
    // v-for的语法规范：警告
    'vue/valid-v-for': 'warn',
    // 未使用变量：警告
    'vue/no-unused-vars': 'warn'
  }
}
