/*
 * @Author: SHEN
 * @Date: 2019-05-20 08:48:31
 * @Last Modified by: SHEN
 * @Last Modified time: 2020-03-23 16:22:50
 */
const utils = require('./utils')
const config = require('./config') // 基本配置的参数
const merge = require('webpack-merge')
const path = require('path')
const HtmlWebpackPlugin = require('html-webpack-plugin')
let baseWebpackConfig = require('./webpack.config')

const HOST = process.env.HOST
const PORT = process.env.PORT && Number(process.env.PORT) // 读取系统环境变量的port

console.log(utils.styleLoaders({ sourceMap: config.dev.cssSourceMap, usePostCSS: true }))
console.log(JSON.stringify(utils.styleLoaders({ sourceMap: config.dev.cssSourceMap, usePostCSS: true })))

let devWebpackConfig = merge(baseWebpackConfig, {
  mode: 'development',
  entry: './src/main.js',
  output: {
    chunkFilename: utils.assetsPath('js/[name].[id].js')
  },
  // cheap-module-eval-source-map is faster for development
  devtool: config.dev.devtool,
  module: {
    // 对一些独立的css文件以及它的预处理文件做一个编译
    rules: utils.styleLoaders({ sourceMap: config.dev.cssSourceMap, usePostCSS: true })
  },
  devServer: {
    clientLogLevel: 'warning', // console 控制台显示的消息，可能的值有 none, error, warning 或者 info
    historyApiFallback: {
      rewrites: [
        { from: /.*/, to: path.posix.join(config.dev.assetsPublicPath, 'index.html') },
      ],
    },
    hot: true, // 开启热模块加载
    contentBase: false, // since we use CopyWebpackPlugin.
    compress: true,
    host: HOST || config.dev.host,
    port: PORT || config.dev.port,
    open: config.dev.autoOpenBrowser,
    overlay: config.dev.errorOverlay
      ? { warnings: false, errors: true }
      : false,
    publicPath: config.dev.assetsPublicPath,
    proxy: config.dev.proxyTable, // 代理设置
    quiet: true, // necessary for FriendlyErrorsPlugin
    watchOptions: { //  启用 Watch 模式。这意味着在初始构建之后，webpack 将继续监听任何已解析文件的更改
      poll: config.dev.poll, // 通过传递 true 开启 polling，或者指定毫秒为单位进行轮询。默认为false
    }
  },
  plugins: [
    new HtmlWebpackPlugin({
      filename: 'index.html',
      template: 'index.html',
      inject: true
    })
  ],
})

module.exports = devWebpackConfig