/*
 * @Author: SHEN
 * @Date: 2019-05-20 09:00:14
 * @Last Modified by: SHEN
 * @Last Modified time: 2020-04-27 14:54:12
 */

var webpack = require('webpack')
const merge = require('webpack-merge')
const utils = require('./utils')
const config = require('./config') // 基本配置的参数
const path = require('path')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const MiniCssExtractPlugin = require("mini-css-extract-plugin")
const CompressionWebpackPlugin = require('compression-webpack-plugin')
const UglifyJsPlugin = require('uglifyjs-webpack-plugin') // uglifyJs 混淆js插件
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin') // optimize-css-assets-webpack-plugin，用于优化和最小化css资源
let baseWebpackConfig = require('./webpack.config')

const components = require('../components.json')
const entrys = {}
Object.keys(components).forEach(item => {
  entrys[item] = components[item]
})

let prodvWebpackConfig = merge(baseWebpackConfig, {
  mode: 'production',
  entry: entrys,
  output: {
    path: utils.resolve('lib'),
    filename: '[name].js',
    library: 'ss_vue_ui', // 指定的就是你使用require时的模块名
    libraryTarget: 'umd', // libraryTarget会生成不同umd的代码,可以只是commonjs标准的，也可以是指amd标准的，也可以只是通过script标签引入的
    umdNamedDefine: true // 会对 UMD 的构建过程中的 AMD 模块进行命名。否则就使用匿名的 define
  },
  devtool: config.build.productionSourceMap ? config.build.devtool : false,
  performance: {
    maxEntrypointSize: 500 * 1024, // 入口文件超过 500KB 进行警告提示
    maxAssetSize: 500 * 1024 // 资源文件超过 500KB 进行警告提示
  },
  module: {
    rules: utils.styleLoaders({
      sourceMap: config.build.productionSourceMap,
      extract: true,
      usePostCSS: true
    })
  },
  optimization: {
    minimizer: [
      new UglifyJsPlugin({
        cache: true,
        parallel: true,
        sourceMap: true
      }),
      new OptimizeCSSAssetsPlugin({})
    ]
  },
  plugins: [
    new MiniCssExtractPlugin({
      filename: utils.assetsPath('theme/[name].css')
    }),
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"'
      }
    }),

  ],
  externals: {
    vue: 'vue'
  }
})

// 如果启动了report，则通过插件给出webpack构建打包后的产品文件分析报告 npm run build --report
if (process.env.npm_config_report) {
  const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
  prodvWebpackConfig.plugins.push(new BundleAnalyzerPlugin())
}

if (process.env.npm_config_gzip) {
  prodvWebpackConfig.plugins.push(new CompressionWebpackPlugin({
    filename: '[path].gz[query]',
    algorithm: 'gzip',
    test: new RegExp(
      '\\.(js|css)$'
    ),
    threshold: 10240,
    minRatio: 0.8
  }))
}

module.exports = prodvWebpackConfig