/*
 * @Author: SHEN
 * @Date: 2019-05-22 15:33:27
 * @Last Modified by: SHEN
 * @Last Modified time: 2020-03-23 13:52:01
 */

import AnimTimes from '@utils/anim-times'

const plugin = (Vue) => {
  return Vue.directive('anim-scroll', {
    bind: function (el, binding, vnode, oldVnode) {
      vnode.context.$nextTick(() => {
        // 判断绑定的元素是否支持滚动
        const overflowY = document.defaultView.getComputedStyle(el).overflowY
        if (overflowY === 'scroll' || overflowY === 'auto') {
          el.addEventListener('scroll', null)
        }

        // 滚动动画时间
        const animDuration = parseInt(el.getAttribute('scroll-duration') || '0')
        el.animDuration = animDuration

        // 滚动方向
        const direction = el.getAttribute('scroll-direction') || 'vertical'
        el.scrollDirection = direction
      })
    },
    inserted: function (el, binding, vnode) {
    },
    update: function (el, binding, vnode) {
      const doneCallback = binding.value
      el.doneCallback = doneCallback

      const scrollTo = el.getAttribute('scroll-to')

      // console.log('anim scroll call update() ===>', scrollTo)

      /** 停止计时器 */
      if (el.interval) {
        clearInterval(el.interval)
        el.interval = null
      }

      // 相同则不处理
      if (el.scrollTo_ === scrollTo) {
        return
      }

      if (typeof (scrollTo) === 'number') {
        scrollToIndex(el, scrollTo)
      } else if (scrollTo === 'bottom') {
        scrollToBottom(el)
      } else if (scrollTo === 'top') {
        scrollToTop(el)
      } else if (scrollTo === 'left') {
        scrollToLeft(el)
      } else if (scrollTo === 'right') {
        scrollToRight(el)
      } else {}
    },
    unbind(el) {
    }
  })
}

function scrollToTop(el) {
  const from = el.scrollTop
  const to = 0
  scrollToPosition(el, from, to, 'vertical', () => {
    el.scrollTo_ = 'idle'
    el.doneCallback && el.doneCallback('top')
  })
}

function scrollToBottom(el) {
  const height = el.scrollHeight
  scrollToPosition(el, el.scrollTop, height, 'vertical', () => {
    el.scrollTo_ = 'idle'
    el.doneCallback && el.doneCallback('bottom')
  })
}

function scrollToLeft(el) {
  const from = el.scrollLeft
  const to = 0
  scrollToPosition(el, from, to, 'horizontal', () => {
    el.scrollTo_ = 'idle'
    el.doneCallback && el.doneCallback('left')
  })
}

function scrollToRight(el) {
  const width = el.scrollWidth
  scrollToPosition(el, el.scrollLeft, width, 'horizontal', () => {
    el.scrollTo_ = 'idle'
    el.doneCallback && el.doneCallback('right')
  })
}

// function scrollToPosition(el, from, to, direction = 'vertical', callback) {
//   let anim = el.animDuration > 0
//   // 是否有动画
//   if (!anim) {
//     el.scrollTop = to
//     // 无动画，需要在下一个渲染周期内执行回调，避免数据在同一个渲染周期内多次变动而导致无效
//     setTimeout(() => {
//       callback()
//     }, 0)
//     return
//   }

//   var time = el.animDuration
//   var n = 5 // 1000毫秒，改变30次
//   var s = from
//   var e = to
//   var d = (e - s) / n

//   let offset = 0
//   let smoothScroll = () => {
//     let scrollTop = el.scrollTop
//     offset += d
//     console.log(scrollTop, e, scrollTop < e)
//     if (scrollTop < e) {
//       window.requestAnimationFrame(smoothScroll)
//       el.scrollTo({
//         top: offset
//       })
//     }
//   }

//   smoothScroll()
// }

function scrollToPosition(el, from, to, direction = 'vertical', callback) {
  const anim = el.animDuration > 0
  // 是否有动画
  if (!anim) {
    el.scrollTop = to
    // 无动画，需要在下一个渲染周期内执行回调，避免数据在同一个渲染周期内多次变动而导致无效
    setTimeout(() => {
      callback()
    }, 300)

    return
  }

  const time = el.animDuration
  const n = 100 // 1000毫秒，改变30次
  const interval = time / n
  let s = from
  var e = to

  const forward = from < to
  let animTimePasted = 0

  // 由于时间函数的显示，这里 s 值要小于 e，所以如果是反向，则调换这两个值
  if (!forward) {
    const tmp = s
    s = e
    e = tmp
  }

  el.interval = setInterval(() => {
    // console.log('interval')
    const current = AnimTimes.Quint.easeInOut(animTimePasted, s, e, time)
    animTimePasted += interval

    if (current < e) {
      if (direction === 'vertical') {
        el.scrollTop = forward ? current : e - current
      }
      if (direction === 'horizontal') {
        el.scrollLeft = forward ? current : e - current
      }
    } else {
      if (direction === 'vertical') {
        el.scrollTop = forward ? e : s
      }
      if (direction === 'horizontal') {
        el.scrollLeft = forward ? e : s
      }
      clearInterval(el.interval)
      el.interval = null
      callback()
    }
  }, interval)
}

// function scrollToPosition(el, from, to, direction = 'vertical', callback) {
//   let anim = el.animDuration > 0
//   // 是否有动画
//   if (!anim) {
//     el.scrollTop = to
//     // 无动画，需要在下一个渲染周期内执行回调，避免数据在同一个渲染周期内多次变动而导致无效
//     setTimeout(() => {
//       callback()
//     }, 0)
//     return
//   }

//   var time = el.animDuration
//   var n = 30 // 1000毫秒，改变30次
//   var s = from; var e = to; var d = (e - s) / n
//   var left = Math.abs(e - s)
//   el.interval = setInterval(() => {
//     s += d
//     left -= Math.abs(d)
//     // console.log('left', left)
//     console.log(el.scrollTop)
//     if (left > -1) { // -1 ???? why not 0
//       if (direction === 'vertical') {
//         el.scrollTop = s
//       }
//       if (direction === 'horizontal') {
//         el.scrollLeft = s
//       }
//     } else {
//       if (direction === 'vertical') {
//         el.scrollTop = e
//       }
//       if (direction === 'horizontal') {
//         el.scrollLeft = e
//       }
//       clearInterval(el.interval)
//       el.interval = null
//       callback()
//     }
//   }, time / n)
// }

function getElementTop(element) {
  var actualTop = element.offsetTop
  var current = element.offsetParent

  while (current !== null) {
    actualTop += current.offsetTop
    current = current.offsetParent
  }
  return actualTop
}

function getElementLeft(element) {
  var actualLeft = element.offsetLeft
  var current = element.offsetParent

  while (current !== null) {
    actualLeft += current.offsetLeft
    current = current.offsetParent
  }

  return actualLeft
}

function scrollToIndex(el, i, callback) {
  const children = el.children
  if (children.length > 0) {
    if (i > children.length - 1 || i < 0) {
      return
    }
    const element = children[i]

    /** 滚动方向为垂直 */
    if (el.scrollDirection === 'vertical') {
      const top = getElementTop(element) - el.offsetTop
      scrollToPosition(el, el.scrollTop, top, 'vertical', () => {
        callback && callback()
        console.log('完成')
      })
    }
    /** 滚动方向为水平 */
    if (el.scrollDirection === 'horizontal') {
      const left = getElementLeft(element) - el.offsetLeft
      scrollToPosition(el, el.scrollLeft, left, 'horizontal', () => {
        callback && callback()
        console.log('完成')
      })
    }
  }
}

export default plugin
