/*
 * @Author: SHEN
 * @Date: 2019-05-22 15:33:27
 * @Last Modified by: SHEN
 * @Last Modified time: 2020-03-23 13:52:03
 */

const plugin = (Vue) => {
  return Vue.directive('anim-scroll', {
    bind: function (el, binding, vnode, oldVnode) {
      vnode.context.$nextTick(() => {
        // 判断绑定的元素是否支持滚动
        const overflowY = document.defaultView.getComputedStyle(el).overflowY
        if (overflowY === 'scroll' || overflowY === 'auto') {
          el.addEventListener('scroll', null)
        }

        // 滚动动画时间
        const animDuration = parseInt(el.getAttribute('scroll-duration') || '1000')
        el.animDuration = animDuration

        // 滚动方向
        const direction = el.getAttribute('scroll-direction') || 'vertical'
        el.scrollDirection = direction

        //
        el.addEventListener('dosome', () => {
          console.log('-----')
        })
      })
    },
    inserted: function (el) {
    },
    update: function (el, binding, vnode) {
      const scrollTo = binding.value
      // console.log('update ', scrollTo)

      // 相同则不处理
      if (el.scrollTo_ === scrollTo) {
        return
      }

      if (typeof (scrollTo) === 'number') {
        scrollToIndex(el, scrollTo)
      } else if (scrollTo === 'bottom') {
        scrollToBottom(el)
      } else if (scrollTo === 'top') {
        scrollToTop(el)
      } else if (scrollTo === 'left') {
        scrollToLeft(el)
      } else if (scrollTo === 'right') {
        scrollToRight(el)
      } else {}
    },
    unbind(el) {
    }
  })
}

function scrollToTop(el) {
  const from = el.scrollTop
  const to = 0
  scrollToPosition(el, from, to, true, 'vertical', () => {
    el.scrollTo_ = 'idle'
    console.log('滚动完毕 to top')
  })
}

function scrollToBottom(el) {
  const height = el.scrollHeight
  scrollToPosition(el, el.scrollTop, height, true, 'vertical', () => {
    el.scrollTo_ = 'idle'
    console.log('滚动完毕 to bottom')
  })
}

function scrollToLeft(el) {
  const from = el.scrollLeft
  const to = 0
  scrollToPosition(el, from, to, true, 'horizontal', () => {
    el.scrollTo_ = 'idle'
    console.log('滚动完毕 to left')
  })
}

function scrollToRight(el) {
  const width = el.scrollWidth
  scrollToPosition(el, el.scrollLeft, width, true, 'horizontal', () => {
    el.scrollTo_ = 'idle'
    console.log('滚动完毕 to right')
  })
}

function scrollToPosition(el, from, to, anim = false, direction = 'vertical', callback) {
  // clear interval first
  el.interval && clearInterval(el.interval)
  if (!anim) {
    el.scrollTop = to
    return
  }
  var time = el.animDuration
  var n = 10 // 1000毫秒，改变30次
  var s = from; var e = to; var d = (e - s) / n
  var left = Math.abs(e - s)
  var interval = setInterval(() => {
    s += d
    left -= Math.abs(d)
    if (left > 0) {
      if (direction === 'vertical') {
        el.scrollTop = s
      }
      if (direction === 'horizontal') {
        el.scrollLeft = s
      }
    } else {
      if (direction === 'vertical') {
        el.scrollTop = e
      }
      if (direction === 'horizontal') {
        el.scrollLeft = e
      }
      clearInterval(el.interval)
      el.interval = null
      callback()
    }
  }, time / n)
  el.interval = interval
}

function getElementTop(element) {
  var actualTop = element.offsetTop
  var current = element.offsetParent

  while (current !== null) {
    actualTop += current.offsetTop
    current = current.offsetParent
  }
  return actualTop
}

function getElementLeft(element) {
  var actualLeft = element.offsetLeft
  var current = element.offsetParent

  while (current !== null) {
    actualLeft += current.offsetLeft
    current = current.offsetParent
  }

  return actualLeft
}

function scrollToIndex(el, i, callback) {
  const children = el.children
  if (children.length > 0) {
    if (i > children.length - 1 || i < 0) {
      return
    }
    const element = children[i]
    /** 滚动方向为垂直 */
    if (el.scrollDirection === 'vertical') {
      const top = getElementTop(element) - el.offsetTop
      scrollToPosition(el, el.scrollTop, top, true, 'vertical', () => {
        callback && callback()
        console.log('完成')
      })
    }
    /** 滚动方向为水平 */
    if (el.scrollDirection === 'horizontal') {
      const left = getElementLeft(element) - el.offsetLeft
      scrollToPosition(el, el.scrollLeft, left, true, 'horizontal', () => {
        callback && callback()
        console.log('完成')
      })
    }
  }
}

export default plugin
