/*
 * @Author: SHEN
 * @Date: 2019-05-06 09:00:27
 * @Last Modified by: SHEN
 * @Last Modified time: 2019-05-06 12:13:46
 *
 * 对于月份，凡是获取得到的值都是从1开始，即一月份的month = 1，凡是进入Date构造函数的month都是从0开始的
 */

/**
  * 获取日期的年，月，日
  * @param {*} date 日期对象
  */
function getYearMonthDay(date) {
  return {
    year: date.getFullYear(),
    month: date.getMonth() + 1,
    day: date.getDate()
  }
}

/**
 * 获取指定日期的那个月的天数
 * @param {*} year 年，数值，如：2019
 * @param {*} month 月份，数值，一月份 = 1
 */
function getDayCountAtDateMonth(year, month) {
  const date = new Date(year, month, 0)
  return date.getDate()
}

/**
 * 获取指定日期是星期几，星期天 = 0，星期一 = 1
 * @param {*} year 年，数值，如：2019
 * @param {*} month 月份，数值，一月份 = 1
 * @param {*} day 日，1号 = 1
 */
function getDayWeek(year, month, day) {
  const date = new Date(year, month - 1, day)
  return date.getDay()
}

/**
 * 上一个月的当前日的日期
 * @param {*} date 日期对象
 */
function switchNextMonth(date) {
  let { year, month, day } = getYearMonthDay(date)
  month += 1
  if (month > 12) {
    month = 1
    year += 1
  }
  return new Date(year, month - 1, day)
}

/**
 * 下一个月的当前日的日期
 * @param {*} date 日期对象
 */
function switchPrevMonth(date) {
  let { year, month, day } = getYearMonthDay(date)
  month -= 1
  if (month < 0) {
    month = 11
    year -= 1
  }
  return new Date(year, month - 1, day)
}

export {
  getYearMonthDay,
  getDayCountAtDateMonth,
  getDayWeek,
  switchNextMonth,
  switchPrevMonth
}
