/*
 * @Author: SHEN
 * @Date: 2019-04-28 19:35:39
 * @Last Modified by: SHEN
 * @Last Modified time: 2019-04-29 14:02:02
 */
import Popover from './popover.vue'
import { getChildrenByAttribute } from '@utils/dom.js'

function getDockingPosition(targetEl, bubbleEl) {
  bubbleEl = getChildrenByAttribute(bubbleEl, 'popover', true)[0]
  const targetFrame = targetEl.getBoundingClientRect()
  const bubbleFrame = bubbleEl.getBoundingClientRect()
  const px = targetFrame.left + targetFrame.width / 2 - bubbleFrame.width / 2
  const py = targetFrame.top + targetFrame.height
  return { x: px, y: py }
}

const plugin = (Vue) => {
  return Vue.directive('popover', {
    bind: function (el, binding, vnode, oldVnode) {
      const elements = getChildrenByAttribute(el, 'popover', true)
      if (elements.length === 0) {
        return
      }
      const popoverContentEl = elements[0]
      popoverContentEl.style.display = 'none'

      const Comp = Vue.extend(Popover)
      const obj = new Comp({
        el: document.createElement('div')
      })

      document.body.appendChild(obj.$el)

      el.addEventListener('click', (e) => {
        obj.visible = !obj.visible
        if (obj.visible) {
          Vue.nextTick(() => {
            const child = popoverContentEl
            child.style.display = 'block'
            obj.$refs.popover.appendChild(child)
            obj.position = getDockingPosition(el, obj.$el)
          })
        }
      })

      vnode.context.$on('hide', () => {
        obj.visible = false
      })
    },
    inserted: function (el) {
    },
    update: function (el, binding) {
    },
    unbind(el) {
    }
  })
}

export default plugin
