/*
 * @Author: SHEN
 * @Date: 2019-03-21 13:45:49
 * @Last Modified by: SHEN
 * @Last Modified time: 2020-03-23 13:56:36
 */
class Refresh {
  constructor(settings = {}) {
    this.settings = Refresh.mergeSettings(settings)
    this.selector = typeof this.settings.selector === 'string'
      ? document.querySelector(this.settings.selector) : this.settings.selector
    if (this.selector === null) {
      throw new Error('wrong selector')
    }

    this.startY = 0
    this.offsetY = 0
    this.triggleOffset = 60
    this.status = 'pending' // pending/pull/release/refresh
    // this.createRefreshElement()
    // this.prepareRefreshImageList()
    this.addEvents()
  }

  createRefreshElement() {
    const wrap = document.createElement('div')
    const canvas = document.createElement('canvas')
    const tip = document.createElement('span')
    wrap.appendChild(canvas)
    wrap.appendChild(tip)
    this.selector.parentNode.appendChild(wrap)

    wrap.className = 'my-refresh'
    canvas.className = 'icon'
    canvas.width = canvas.clientWidth
    canvas.height = canvas.clientHeight

    tip.className = 'tip'
    tip.innerHTML = this.settings.pullText

    this.drawContext = canvas.getContext('2d')
  }

  prepareRefreshImageList() {
    const images = []
    for (let i = 0; i < 8; i++) {
      const path = `./static/images/${i + 1}.png` // 不能是当前目录的相对路径
      const img = new Image()
      img.src = path
      images.push(img)
    }
    this.images = images
  }

  stepAnimateIcon(offsetY) {
    var i = Math.floor(offsetY * 8 / 60)
    i = i % 8
    const img = this.images[i]
    this.drawContext.clearRect(0, 0, 40, 40)
    this.drawContext.drawImage(img, 0, 0, 40, 40)
  }

  fullAnimateIcon() {
    if (this.interval) {
      clearInterval(this.interval)
    }
    var i = 0
    this.interval = setInterval(() => {
      const img = this.images[i]
      this.drawContext.clearRect(0, 0, 40, 40)
      this.drawContext.drawImage(img, 0, 0, 40, 40)
      if (++i > 7) {
        i = 0
      }
    }, 100)

    setTimeout(() => {
      clearInterval(this.interval)
      this.interval = null

      const selector = this.selector
      selector.style.transition = 'all 300ms ease'
      selector.style.transform = 'translate3d(0, 0, 0)'
    }, 3000)
  }

  updateStatusLabel() {
    const dom = document.querySelector('.my-refresh .tip')
    // pending/pull/release/refresh
    const status = this.status
    if (status === 'pending') {
      dom.innerHTML = this.settings.peddingText
    } else if (status === 'pull') {
      dom.innerHTML = this.settings.pullText
    } else if (status === 'release') {
      dom.innerHTML = this.settings.relaseText
    } else {
      dom.innerHTML = this.settings.refreshText
    }
  }

  addEvents() {
    const selector = this.selector
    this.handleTouchStart = this.handleTouchStart.bind(this)
    this.handleTouchMove = this.handleTouchMove.bind(this)
    this.handleTouchEnd = this.handleTouchEnd.bind(this)
    selector.addEventListener('touchstart', this.handleTouchStart)
    selector.addEventListener('touchmove', this.handleTouchMove)
    selector.addEventListener('touchend', this.handleTouchEnd)
  }

  handleTouchStart(e) {
    this.startY = e.touches[0].pageY
    this.status = 'pending'
  }

  handleTouchMove(e) {
    const selector = this.selector
    const offsetY = (e.touches[0].pageY - this.startY) / 2
    if (selector.scrollTop === 0 && offsetY > 0) {
      selector.style.transition = 'all 0ms ease'
      selector.style.transform = `translate3d(0, ${offsetY}px, 0)`

      this.status = 'pull'
      if (offsetY > this.triggleOffset) {
        this.status = 'release'
      }

      // this.stepAnimateIcon(offsetY)
      this.updateStatusLabel()
    }
  }

  handleTouchEnd(e) {
    const selector = this.selector
    selector.style.transition = 'all 300ms ease'

    if (this.status === 'release') {
      selector.style.transform = `translate3d(0, 60px, 0)`
      this.fullAnimateIcon()
    } else {
      selector.style.transform = `translate3d(0, 0, 0)`
    }
    this.status = 'refresh'
    this.updateStatusLabel()
  }

  static mergeSettings(settings) {
    const _settings = {
      selector: '.list',
      peddingText: 'Hello',
      pullText: '下拉以刷新页面',
      relaseText: '释放以刷新页面',
      refreshText: '刷新...'
    }

    const userSttings = settings
    for (const attrname in userSttings) {
      _settings[attrname] = userSttings[attrname]
    }
    return _settings
  }
}

export default Refresh
