/*
 * @Author: SHEN
 * @Date: 2019-04-22 23:22:38
 * @Last Modified by: SHEN
 * @Last Modified time: 2019-06-03 08:58:20
 *
 * 滚动位置保持
 */

import throttle from '@utils/throttle.js'
const context = '@@ScrollMaintain'

var getScrollTop = function(element) {
  if (element === window) {
    return Math.max(window.pageYOffset || 0, document.documentElement.scrollTop)
  }
  return element.scrollTop
}

var bindEvents = function () {
  if (this.binded) {
    return
  }
  this.binded = true
  this.scrollListener = throttle(handleScroll.bind(this), 200)
  this.el.addEventListener('scroll', this.scrollListener)
}

var handleScroll = function () {
  const scrollTop = getScrollTop(this.el)
  if (window && window.scrollPositions) {
    window.scrollPositions[this.name] = scrollTop
  } else {
    window.scrollPositions = {}
    window.scrollPositions[this.name] = scrollTop
  }
}

var plugin = {
  bind: function (el, binding, vnode, oldVnode) {
    el[context] = {
      el: el,
      vm: vnode.context,
      expression: binding.value,
      name: binding.arg
    }

    vnode.context.$nextTick(() => {
      if (vnode.context._isMounted) {
        bindEvents.call(el[context], arguments)
      }
    })
  },
  // 当被绑定的元素插入到 DOM 中时
  inserted: function (el) {
    if (window && window.scrollPositions) {
      const scrollTop = window.scrollPositions[el[context].name]
      scrollTop && (el.scrollTop = scrollTop)
    }
  },
  update: function (el, binding) {
    if (window && window.scrollPositions) {
      const scrollTop = window.scrollPositions[el[context].name]
      scrollTop && (el.scrollTop = scrollTop)
    }
  },
  unbind(el) {
    el.removeEventListener('scroll', el[context].scrollListener)
  }
}

export default plugin
