import Vue from 'vue'
import merge from '../merge'
import PopupManager from './popup-manager'

let idSeed = 1

const getDOM = function(dom) {
  if (dom.nodeType === 3) { // Element 或者 Attr 中实际的文字
    dom = dom.nextElementSibling || dom.nextSibling
    getDOM(dom)
  }
  return dom
}

export default {
  props: {
    value: {
      type: Boolean,
      default: false
    },
    transition: {
      type: String,
      default: ''
    },
    openDelay: {
      type: Number,
      default: 0
    },
    closeDelay: {
      type: Number,
      default: 0
    },
    zIndex: {
      type: Number,
      default: 0
    },
    modal: {
      type: Boolean,
      default: false
    },
    modalFade: {
      type: Boolean,
      default: true
    },
    modalClass: {
      type: String,
      default: ''
    },
    lockScroll: {
      type: Boolean,
      default: true
    },
    closeOnPressEscape: {
      type: Boolean,
      default: false
    },
    closeOnClickModal: {
      type: Boolean,
      default: false
    }
  },

  data() {
    return {
      // 是否已经显示
      opened: false,
      bodyOverflow: null,
      bodyPaddingRight: null,
      rendered: false
    }
  },

  created() {
  },

  beforeMount() {
    this._popupId = 'popup-' + idSeed++
    PopupManager.register(this._popupId, this)
  },

  beforeDestroy() {
    PopupManager.deregister(this._popupId)
    PopupManager.closeModal(this._popupId)
    // if (this.modal && this.bodyOverflow !== null && this.bodyOverflow !== 'hidden') {
    //   document.body.style.overflow = this.bodyOverflow
    //   document.body.style.paddingRight = this.bodyPaddingRight
    // }
    // this.bodyOverflow = null
    // this.bodyPaddingRight = null
  },

  watch: {
    value(val) {
      if (val) {
        if (this._opening) return
        if (!this.rendered) {
          this.rendered = true
          Vue.nextTick(() => {
            this.open()
          })
        } else {
          this.open()
        }
      } else {
        this.close()
      }
    }
  },

  methods: {
    open() {
      if (!this.rendered) {
        this.rendered = true
        this.$emit('input', true)
      }

      const props = this.$props // merge({}, this, this.$props)

      if (this._closeTimer) {
        clearTimeout(this._closeTimer)
        this._closeTimer = null
      }
      clearTimeout(this._openTimer)

      const openDelay = Number(props.openDelay)
      if (openDelay > 0) {
        this._openTimer = setTimeout(() => {
          this._openTimer = null
          this.doOpen(props)
        }, openDelay)
      } else {
        this.doOpen(props)
      }
    },

    doOpen(props) {
      if (this.opened) {
        return
      }

      // 使用 vue-popup 的组件，如果需要和父组件通信显示的状态，应该使用 value，它是一个 prop，
      // 这样在父组件中用 v-model 即可；否则可以使用 visible，它是一个 data
      this.visible = true
      this.$emit('input', true)

      const dom = getDOM(this.$el)

      const modal = props.modal

      if (modal) {
        if (this._closing) {
          PopupManager.closeModal(this._popupId)
          this._closing = false
        }
        PopupManager.openModal(this._popupId, PopupManager.nextZIndex(), dom, props.modalClass, props.modalFade)
      }

      dom.style.zIndex = PopupManager.nextZIndex()
      this.opened = true
    },

    close() {
      if (this._openTimer !== null) {
        clearTimeout(this._openTimer)
        this._openTimer = null
      }
      clearTimeout(this._closeTimer)

      const closeDelay = Number(this.closeDelay)

      if (closeDelay > 0) {
        this._closeTimer = setTimeout(() => {
          this._closeTimer = null
          this.doClose()
        }, closeDelay)
      } else {
        this.doClose()
      }
    },

    doClose() {
      this.visible = false
      this.$emit('input', false)
      this._closing = true

      this.onClose && this.onClose()

      // if (this.lockScroll) {
      //   setTimeout(() => {
      //     if (this.modal && this.bodyOverflow !== 'hidden') {
      //       document.body.style.overflow = this.bodyOverflow
      //       document.body.style.paddingRight = this.bodyPaddingRight
      //     }
      //     this.bodyOverflow = null
      //     this.bodyPaddingRight = null
      //   }, 200)
      // }

      this.opened = false

      if (!this.transition) {
        this.doAfterClose()
      }
    },

    doAfterClose() {
      PopupManager.closeModal(this._popupId)
      this._closing = false
    }
  }
}

export { PopupManager }
